﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using SocialEducatorAvalonia.Models;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Reactive.Linq;
using SocialEducatorAvalonia.Utilities;
using System.Collections.Immutable;
using Microsoft.EntityFrameworkCore;

namespace SocialEducatorAvalonia.ViewModels
{
    internal partial class StudentsViewModel : ViewModelBase
    {
        #region arrays for showing on page
        /// <summary>
        /// Отображаемая коллекция групп на странице 
        /// </summary>
        readonly ImmutableArray<string> _groupList;
        public ImmutableArray<string> GroupList { get => _groupList; }

        /// <summary>
        /// Перечисление статусов (Без Обучающийся)
        /// </summary>
        readonly ImmutableArray<string> _statusList;
        public ImmutableArray<string> StatusList { get => _statusList; }

        /// <summary>
        /// Отображаемая на странице коллекция
        /// </summary>
        [ObservableProperty]
        ObservableCollection<Student> _studentsList = [];
        #endregion

        #region selected items on page
        /// <summary>
        /// Введенная группа на странице
        /// </summary>
        [ObservableProperty]
        string? _selectedGroup = null;

        /// <summary>
        /// Введенное ФИО на странице
        /// </summary>
        [ObservableProperty]
        static string? _selectedFIO = null;

        /// <summary>
        /// Введенный статус на странице
        /// </summary>
        [ObservableProperty]
        string? _selectedStatus = null;
        #endregion

        /// <summary>
        /// перечисление студентов в кеше
        /// </summary>
        List<Student> _studentsStorage = [];

        #region pagination
        /// <summary>
        /// Текущая позиция по выборке из 25
        /// </summary>
        [ObservableProperty]
        int _currentPage = 1;

        /// <summary>
        /// Всего элементов, с условием что мы делим на 25
        /// </summary>
        [ObservableProperty]
        int _pageSize = 0;

        /// <summary>
        /// Обновление отображаемой коллекции
        /// </summary>
        private void UpdateViewCollection()
        {
            int skip = (CurrentPage - 1) * 25;
            StudentsList = new ObservableCollection<Student>(_studentsStorage.Skip(skip).Take(25));
        }

        /// <summary>
        /// Команда срабатывающая на кнопку -> 
        /// </summary>
        [RelayCommand]
        void GoNextPage()
        {
            if (CurrentPage + 1 <= PageSize)
            {
                CurrentPage++;
                UpdateViewCollection();
            }
        }

        /// <summary>
        /// Команда срабатывающая на кнопку <-
        /// </summary>
        [RelayCommand]
        void GoPreviousPage()
        {
            if (CurrentPage - 1 >= 1)
            {
                CurrentPage--;
                UpdateViewCollection();
            }
        }

        /// <summary>
        /// Обновить элементы пагинации
        /// </summary>
        private void UpdateCountOfPages()
        {
            PageSize = (int)Math.Ceiling((decimal)_studentsStorage.Count / (decimal)25);
            CurrentPage = 1;
        }

        /// <summary>
        /// Обновление хранимой в кеш коллекции + страничной
        /// </summary>
        /// <param name="collection">Коллекция из запроса</param>
        private void UpdateCollections(ref ObservableCollection<Student> collection)
        {
            if (collection.Count > 25)
            {
                _studentsStorage = [.. collection];
                UpdateCountOfPages();
                UpdateViewCollection();
            }
            else
            {
                StudentsList = collection;
                PageSize = 1;
            }
        }
        #endregion

        /// <summary>
        /// Отправка аргументов в DataEditorUtilities
        /// </summary>
        void SentArguments()
        {
            DataEditorUtilities.SelectedFIO = SelectedFIO;
            DataEditorUtilities.SelectedGroup = SelectedGroup;
            DataEditorUtilities.SelectedStatus = SelectedStatus;
        }

        partial void OnSelectedGroupChanged(string? value)
        {
            SelectedGroup = value;

            SentArguments();

            var studList = DataEditorUtilities.FindByGroup(value, ref _studentsStorage, ref _studentsList);

            if (studList != null)
                UpdateCollections(ref studList);
        }

        partial void OnSelectedStatusChanged(string? value)
        {
            SelectedStatus = value;

            SentArguments();

            var studentsList = DataEditorUtilities.FindByStatus(value, ref _studentsList, ref _studentsStorage);

            if (studentsList is not null)
                UpdateCollections(ref studentsList);
        }

        partial void OnSelectedFIOChanged(string? value)
        {
            SelectedFIO = value;

            SentArguments();

            var students_List = DataEditorUtilities.FindUserByFIO(value, ref _studentsList, ref _studentsStorage);

            if (students_List is not null)
                UpdateCollections(ref students_List);
        }

        /// <summary>
        /// Конструктор для создания страницы
        /// </summary>
        public StudentsViewModel()
        {
            _groupList = GroupDict.Keys.ToImmutableArray();

            // Берем все элементы кроме "Обучающийся", т.к. часто встречается
            _statusList = StatusDict.Keys.Where((v, i) => v != "Обучается").ToImmutableArray();

            _studentsStorage = DbContext.StudentsActiveAndGraduations.Include(x => x.StudentStatus).Include(x => x.StudentGroup).Take(200).AsEnumerable()
                .Select(x =>
                {
                    string[] spilledFio = x.Fio.Split(' ');
                    return new Student
                    {
                        StudentId = x.StudentId,
                        Fio = x.Fio,
                        Surname = spilledFio.Length > 0 ? spilledFio[0] : string.Empty,
                        Name = spilledFio.Length > 1 ? spilledFio[1] : string.Empty,
                        Patronymic = spilledFio.Length > 2 ? spilledFio[2] : string.Empty,
                        AdmissionDate = x.AdmissionDate,
                        StudentGroupId = x.StudentGroupId,
                        StudentStatusId = x.StudentStatusId,
                        StudentGroup = x.StudentGroup,
                        StudentStatus = x.StudentStatus
                    };
                }).ToList();

            StudentsList = new ObservableCollection<Student>(_studentsStorage.Take(25));

            UpdateCountOfPages();
        }

        [RelayCommand]
        void GoToCreationStudentPage()
        {
            MainWindowViewModel.AddDataTab = 0;
            MainVM.ChangeView(MainWindowViewModel.ListItems[1]);
        }

        [RelayCommand]
        void GoToEditPage(Student currentStudent) => PageSwitcher.View = new EditStudentViewModel(currentStudent);
    }
}